using System;
using System.Windows.Media;
using Styx;
using Styx.Common;
using Styx.Plugins;
using Styx.WoWInternals;

namespace LuckyDo
{
    internal class LuckyDo : HBPlugin
    {
        public enum SpellPrompt
        {
            //Ids found here
            //http://www.wowhead.com/spells=0?filter=na=Bonus;cr=84:109:16;crs=1:6:6

            Unkown = 0,

            //Draenor
            DrovTheRuiner = 178847,
            TarlnaTheAgeless = 178851,
            Rukhmar = 178849,
            SupremeLordKazzak = 188985,

            //Highmaul
            KargathBladefist = 177521,
            TheButcher = 177522,
            Tectus = 177523,
            Brackenspore = 177524,
            TwinOgron = 177525,
            Koragh = 177526,
            ImperatorMargok = 177528,

            //Blackrock Foundry
            Gruul = 177529,
            Oregorger = 177530,
            BeastlordDarmac = 177536,
            FlamebenderKagraz = 177534,
            HansgarFranzok = 177533,
            OperatorThogar = 177537,
            TheBlastFurnace = 177531,
            Kromog = 177535,
            TheIronMaidens = 177538,
            Blackhand = 177539,

            //Hellfire Citadel
            HellfireAssault = 188972,
            IronReaver = 188973,
            Kormrok = 188974,
            HellfireHighCouncil = 188975,
            KilroggDeadeye = 188976,
            Gorefiend = 188977,
            ShadowLordIskar = 188978,
            Socrethar = 188979,
            FelLordZakuun = 188980,
            Xhulhorac = 188981,
            TyrantVelhari = 188982,
            Mannoroth = 188983,
            Archimonde = 188984
        }

        private static bool _isAttached;

        public override string Name => "LuckyDo";

        public override string Author => "Nuok";
        public override Version Version => new Version(4, 0, 4);
        public override bool WantButton => true;

        public bool CheckRaidMode
        {
            get
            {
                switch (StyxWoW.Me.GroupInfo.RaidDifficultyId)
                {
                    case 14:
                    {
                        Log("Difficulty is Normal, Do we want to roll? {0}",
                            LuckySettings.Instance.RaidDifficulty.UseNormal);
                        return LuckySettings.Instance.RaidDifficulty.UseNormal;
                    }
                    case 15:
                    {
                        Log("Difficulty is Heroic, Do we want to roll? {0}",
                            LuckySettings.Instance.RaidDifficulty.UseHeroic);
                        return LuckySettings.Instance.RaidDifficulty.UseHeroic;
                    }
                    case 16:
                    {
                        Log("Difficulty is Mythic, Do we want to roll? {0}",
                            LuckySettings.Instance.RaidDifficulty.UseMythic);
                        return LuckySettings.Instance.RaidDifficulty.UseMythic;
                    }
                    case 17:
                    {
                        Log("Difficulty is LFR, Do we want to roll? {0}",
                            LuckySettings.Instance.RaidDifficulty.UseLFR);
                        return LuckySettings.Instance.RaidDifficulty.UseLFR;
                    }
                }
                Log("Didn't recognize RaidDifficultyId {0}, will return true as failsafe.",
                    StyxWoW.Me.GroupInfo.RaidDifficultyId);
                return true;
            }
        }

        public override void Pulse()
        {
        }

        public override void OnButtonPress()
        {
            string logTag = $"{Name} v{Version}";
            var window = new ConfigWindow(Name, logTag, Author, 350, 400);
            window.Show();
            window.Closed += ConfigWindowOnClosed;
        }

        private void ConfigWindowOnClosed(object sender, EventArgs e)
        {
            LuckySettings.Instance.Save();
            LuckySettings.Instance.DraenorSettings.Save();
            LuckySettings.Instance.RaidDifficulty.Save();
            LuckySettings.Instance.Tier17Settings.Save();
            Log("Settings Saved");
        }

        private void Roll(object sender, LuaEventArgs args)
        {
            var prompt = (SpellPrompt) Lua.ParseLuaValue<int>(args.Args[0].ToString());
            var roll = CheckSetting(prompt);
            var raidMode = CheckRaidMode;
            Debug("SPELL_CONFIRMATION_PROMPT {0} ({1}) ShouldRoll? {2}", prompt, (int) prompt, roll);
            if (args.Args == null || !raidMode || !roll) return;
            Log("Rolling {0}", prompt);
            Lua.DoString($"AcceptSpellConfirmationPrompt({(int) prompt})");
        }

        public override void OnEnable()
        {
            if (_isAttached) return;
            Log("Welcome Friend of the Grummels");
            Lua.Events.AttachEvent("BONUS_ROLL_RESULT", LogResult);
            Lua.Events.AttachEvent("SPELL_CONFIRMATION_PROMPT", Roll);
            _isAttached = true;
        }

        public override void OnDisable()
        {
            if (!_isAttached) return;
            Log("Farewell Friend of the Grummels");
            Lua.Events.DetachEvent("BONUS_ROLL_RESULT", LogResult);
            Lua.Events.DetachEvent("SPELL_CONFIRMATION_PROMPT", Roll);
            _isAttached = false;
        }

        public void Log(string format, params object[] args)
        {
            Logging.Write(Colors.Goldenrod, "LuckyDo: " + format, args);
        }

        public void Debug(string format, params object[] args)
        {
            Logging.WriteVerbose(Colors.Red, "LuckyDoDebug: " + format, args);
        }

        private void LogResult(object sender, LuaEventArgs args)
        {
            Log("Are you a LuckyDo?");
            if (args.Args == null) return;
            if (Convert.ToString(args.Args[0]) == "money")
            {
                Log("Sorry Friend More Gold :(");
                return;
            }

            var itemWon = Lua.GetReturnVal<string>($"return GetItemInfo({Convert.ToString(args.Args[1])})", 0);
            if (itemWon != null)
            {
                Log("You won {0}, you LuckyDo", itemWon);
            }
        }

        public bool CheckSetting(SpellPrompt prompt)
        {
            switch (prompt)
            {
                //Draenor
                case SpellPrompt.DrovTheRuiner:
                    return LuckySettings.Instance.DraenorSettings.DrovTheRuiner;
                case SpellPrompt.TarlnaTheAgeless:
                    return LuckySettings.Instance.DraenorSettings.TarlnaTheAgeless;
                case SpellPrompt.Rukhmar:
                    return LuckySettings.Instance.DraenorSettings.Rukhmar;

                //Highmaul
                case SpellPrompt.KargathBladefist:
                    return LuckySettings.Instance.Tier17Settings.KargathBladefist;
                case SpellPrompt.TheButcher:
                    return LuckySettings.Instance.Tier17Settings.TheButcher;
                case SpellPrompt.Tectus:
                    return LuckySettings.Instance.Tier17Settings.Tectus;
                case SpellPrompt.Brackenspore:
                    return LuckySettings.Instance.Tier17Settings.Brackenspore;
                case SpellPrompt.TwinOgron:
                    return LuckySettings.Instance.Tier17Settings.TwinOgron;
                case SpellPrompt.Koragh:
                    return LuckySettings.Instance.Tier17Settings.Koragh;
                case SpellPrompt.ImperatorMargok:
                    return LuckySettings.Instance.Tier17Settings.ImperatorMargok;

                //Blackrock Foundry
                case SpellPrompt.Gruul:
                    return LuckySettings.Instance.Tier17Settings.Gruul;
                case SpellPrompt.Oregorger:
                    return LuckySettings.Instance.Tier17Settings.Oregorger;
                case SpellPrompt.BeastlordDarmac:
                    return LuckySettings.Instance.Tier17Settings.BeastlordDarmac;
                case SpellPrompt.FlamebenderKagraz:
                    return LuckySettings.Instance.Tier17Settings.FlamebenderKagraz;
                case SpellPrompt.HansgarFranzok:
                    return LuckySettings.Instance.Tier17Settings.HansgarFranzok;
                case SpellPrompt.OperatorThogar:
                    return LuckySettings.Instance.Tier17Settings.OperatorThogar;
                case SpellPrompt.TheBlastFurnace:
                    return LuckySettings.Instance.Tier17Settings.TheBlastFurnace;
                case SpellPrompt.Kromog:
                    return LuckySettings.Instance.Tier17Settings.Kromog;
                case SpellPrompt.TheIronMaidens:
                    return LuckySettings.Instance.Tier17Settings.TheIronMaidens;
                case SpellPrompt.Blackhand:
                    return LuckySettings.Instance.Tier17Settings.Blackhand;

                //Hellfire
                case SpellPrompt.Gorefiend:
                    return LuckySettings.Instance.Tier18Settings.Gorefiend;
                case SpellPrompt.IronReaver:
                    return LuckySettings.Instance.Tier18Settings.IronReaver;
                case SpellPrompt.Kormrok:
                    return LuckySettings.Instance.Tier18Settings.Kormrok;
                case SpellPrompt.HellfireHighCouncil:
                    return LuckySettings.Instance.Tier18Settings.HellfireHighCouncil;
                case SpellPrompt.KilroggDeadeye:
                    return LuckySettings.Instance.Tier18Settings.KilroggDeadeye;
                case SpellPrompt.HellfireAssault:
                    return LuckySettings.Instance.Tier18Settings.HellfireAssault;
                case SpellPrompt.ShadowLordIskar:
                    return LuckySettings.Instance.Tier18Settings.ShadowLordIskar;
                case SpellPrompt.Socrethar:
                    return LuckySettings.Instance.Tier18Settings.SocretharTheEternal;
                case SpellPrompt.TyrantVelhari:
                    return LuckySettings.Instance.Tier18Settings.TyrantVelhari;
                case SpellPrompt.Archimonde:
                    return LuckySettings.Instance.Tier18Settings.Archimonde;
                case SpellPrompt.FelLordZakuun:
                    return LuckySettings.Instance.Tier18Settings.FelLordZakuun;
                case SpellPrompt.Xhulhorac:
                    return LuckySettings.Instance.Tier18Settings.Xhulhorac;
                case SpellPrompt.Mannoroth:
                    return LuckySettings.Instance.Tier18Settings.Mannoroth;
            }
            Log("No prompt found for {0}", prompt);
            return false;
        }
    }
}