﻿using System.ComponentModel;
using System.IO;
using Styx;
using Styx.Common;
using Styx.Helpers;

namespace LuckyDo
{
    internal class LuckySettings : Settings
    {
        private static LuckySettings _instance;
        private Draenor _draenor;
        private DungonDifficulty _dungonDifficulty;
        private Tier17 _tier17;
        private Tier18 _tier18;

        public LuckySettings()
            : base(Path.Combine(CharacterSettingsPath, "LuckyDooSettings.xml"))
        {
        }

        public static string CharacterSettingsPath
        {
            get
            {
                string settingsDirectory = Path.Combine(Utilities.AssemblyDirectory, "Settings");
                return Path.Combine(Path.Combine(settingsDirectory, "LuckyDoo", StyxWoW.Me.RealmName), StyxWoW.Me.Name);
            }
        }

        public static LuckySettings Instance => _instance ?? (_instance = new LuckySettings());

        [Browsable(false)]
        public Draenor DraenorSettings => _draenor ?? (_draenor = new Draenor());

        [Browsable(false)]
        public Tier17 Tier17Settings => _tier17 ?? (_tier17 = new Tier17());

        [Browsable(false)]
        public Tier18 Tier18Settings => _tier18 ?? (_tier18 = new Tier18());

        [Browsable(false)]
        public DungonDifficulty RaidDifficulty => _dungonDifficulty ?? (_dungonDifficulty = new DungonDifficulty());

        [Browsable(false)]
        internal class Draenor : Settings
        {
            public Draenor() :
                base(Path.Combine(CharacterSettingsPath, "Draenor.xml"))
            {
            }

            #region Draenor

            [Setting]
            [Category("Draenor")]
            [DisplayName("Drov the Ruiner")]
            [Styx.Helpers.DefaultValue(false)]
            public bool DrovTheRuiner { get; set; }

            [Setting]
            [Category("Draenor")]
            [DisplayName("Tarlna the Ageless")]
            [Styx.Helpers.DefaultValue(false)]
            public bool TarlnaTheAgeless { get; set; }

            [Setting]
            [Category("Draenor")]
            [DisplayName("Rukhmar")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Rukhmar { get; set; }

            [Setting]
            [Category("Draenor")]
            [DisplayName("Supreme Lord Kazzak")]
            [Styx.Helpers.DefaultValue(false)]
            public bool SupremeLordKazzak { get; set; }

            #endregion
        }

        [Browsable(false)]
        internal class Tier18 : Settings
        {
            public Tier18() :
                base(Path.Combine(CharacterSettingsPath, "Tier18.xml"))
            {
            }

            #region Draenor

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Hellfire Assault")]
            [Styx.Helpers.DefaultValue(false)]
            public bool HellfireAssault { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Iron Reaver")]
            [Styx.Helpers.DefaultValue(false)]
            public bool IronReaver { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Kormrok")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Kormrok { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Hellfire High Council")]
            [Styx.Helpers.DefaultValue(false)]
            public bool HellfireHighCouncil { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Kilrogg Deadeye")]
            [Styx.Helpers.DefaultValue(false)]
            public bool KilroggDeadeye { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Gorefiend")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Gorefiend { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Fel Lord Zakuun")]
            [Styx.Helpers.DefaultValue(false)]
            public bool FelLordZakuun { get; set; }


            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Shadow-Lord Iskar")]
            [Styx.Helpers.DefaultValue(false)]
            public bool ShadowLordIskar { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Socrethar the Eternal")]
            [Styx.Helpers.DefaultValue(false)]
            public bool SocretharTheEternal { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Xhul'horac")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Xhulhorac { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Tyrant Velhari")]
            [Styx.Helpers.DefaultValue(false)]
            public bool TyrantVelhari { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Mannoroth")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Mannoroth { get; set; }

            [Setting]
            [Category("Hellfire Citadel")]
            [DisplayName("Archimonde")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Archimonde { get; set; }

            #endregion
        }

        internal class DungonDifficulty : Settings
        {
            public DungonDifficulty() :
                base(Path.Combine(CharacterSettingsPath, "DungonDifficulty.xml"))
            {
            }

            [Setting]
            [Category("Dungon Difficulty")]
            [DisplayName("Normal Raid")]
            [Styx.Helpers.DefaultValue(true)]
            public bool UseNormal { get; set; }

            [Setting]
            [Category("Dungon Difficulty")]
            [DisplayName("Heroic Raid")]
            [Styx.Helpers.DefaultValue(true)]
            public bool UseHeroic { get; set; }

            [Setting]
            [Category("Dungon Difficulty")]
            [DisplayName("Mythic Raid")]
            [Styx.Helpers.DefaultValue(true)]
            public bool UseMythic { get; set; }

            [Setting]
            [Category("Dungon Difficulty")]
            [DisplayName("Looking For Raid")]
            [Styx.Helpers.DefaultValue(true)]
            public bool UseLFR { get; set; }
        }
        [Browsable(false)]
        internal class Tier17 : Settings
        {
            public Tier17() :
                base(Path.Combine(CharacterSettingsPath, "Tier17.xml"))
            {
            }

            #region Highmaul

            [Setting]
            [Category("Highmaul")]
            [DisplayName("Kargath Bladefist")]
            [Styx.Helpers.DefaultValue(false)]
            public bool KargathBladefist { get; set; }

            [Setting]
            [Category("Highmaul")]
            [DisplayName("The Butcher")]
            [Styx.Helpers.DefaultValue(false)]
            public bool TheButcher { get; set; }

            [Setting]
            [Category("Highmaul")]
            [DisplayName("Tectus")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Tectus { get; set; }

            [Setting]
            [Category("Highmaul")]
            [DisplayName("Brackenspore")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Brackenspore { get; set; }

            [Setting]
            [Category("Highmaul")]
            [DisplayName("Twin Ogron")]
            [Styx.Helpers.DefaultValue(false)]
            public bool TwinOgron { get; set; }

            [Setting]
            [Category("Highmaul")]
            [DisplayName("Ko'ragh")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Koragh { get; set; }

            [Setting]
            [Category("Highmaul")]
            [DisplayName("Imperator Mar'gok")]
            [Styx.Helpers.DefaultValue(false)]
            public bool ImperatorMargok { get; set; }

            #endregion

            #region Blackrock Foundry

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("Gruul")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Gruul { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("Oregorger")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Oregorger { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("Hans'gar and Franzok")]
            [Styx.Helpers.DefaultValue(false)]
            public bool HansgarFranzok { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("Beastlord Darmac")]
            [Styx.Helpers.DefaultValue(false)]
            public bool BeastlordDarmac { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("Flamebender Ka'graz")]
            [Styx.Helpers.DefaultValue(false)]
            public bool FlamebenderKagraz { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("Operator Thogar")]
            [Styx.Helpers.DefaultValue(false)]
            public bool OperatorThogar { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("The Blast Furnace")]
            [Styx.Helpers.DefaultValue(false)]
            public bool TheBlastFurnace { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("Kromog")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Kromog { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("The Iron Maidens")]
            [Styx.Helpers.DefaultValue(false)]
            public bool TheIronMaidens { get; set; }

            [Setting]
            [Category("Blackrock Foundry")]
            [DisplayName("Blackhand")]
            [Styx.Helpers.DefaultValue(false)]
            public bool Blackhand { get; set; }

            #endregion
        }
    }
}